import { RPC, RequestOf, ResponseOf } from '@previewjs/api';
import { Reader } from '@previewjs/vfs';
import * as vite from 'vite';
import vite__default from 'vite';
import { ValueType, TypeAnalyzer, CollectedTypes } from '@previewjs/type-analyzer';
import ts from 'typescript';
import * as express from 'express';

type PackageDependencies = Record<string, {
    /** @deprecated */
    majorVersion: number;
    readInstalledVersion(): Promise<string | null>;
}>;

interface FrameworkPluginFactory {
    isCompatible(dependencies: PackageDependencies): Promise<boolean>;
    create(): Promise<FrameworkPlugin>;
}
interface FrameworkPlugin {
    readonly pluginApiVersion?: number;
    readonly name: string;
    readonly defaultWrapperPath: string;
    readonly previewDirPath: string;
    readonly transformReader?: (reader: Reader, rootDirPath: string) => Reader;
    readonly tsCompilerOptions?: Partial<ts.CompilerOptions>;
    readonly specialTypes?: Record<string, ValueType>;
    readonly viteConfig: () => vite__default.UserConfig;
    readonly incompatibleVitePlugins?: string[];
    readonly detectComponents: (typeAnalyzer: TypeAnalyzer, absoluteFilePaths: string[]) => Promise<Component[]>;
}
interface Component {
    readonly absoluteFilePath: string;
    readonly name: string;
    readonly offsets: Array<[start: number, end: number]>;
    readonly info: {
        kind: "component";
        readonly exported: boolean;
        readonly analyze: () => Promise<ComponentAnalysis>;
    } | {
        kind: "story";
        readonly associatedComponent: {
            readonly absoluteFilePath: string;
            readonly name: string;
        } | null;
    };
}
interface ComponentAnalysis {
    propsType: ValueType;
    types: CollectedTypes;
}

type RegisterRPC = <E extends RPC<any, any>>(endpoint: E, handler: RequestHandler<RequestOf<E>, ResponseOf<E>>) => void;
type RequestHandler<Req, Res> = (request: Req) => Promise<Res>;

type SetupPreviewEnvironment = (options: {
    versionCode?: string;
    registerRPC: RegisterRPC;
    workspace: Workspace;
}) => Promise<{
    middlewares?: express.RequestHandler[];
}>;

declare function setupFrameworkPlugin({ rootDirPath, frameworkPluginFactories, }: {
    rootDirPath: string;
    frameworkPluginFactories: FrameworkPluginFactory[];
}): Promise<FrameworkPlugin | null>;

declare function createWorkspace({ rootDirPath, reader, frameworkPlugin, logLevel, setupEnvironment, }: {
    rootDirPath: string;
    frameworkPlugin: FrameworkPlugin;
    logLevel: vite.LogLevel;
    reader: Reader;
    setupEnvironment?: SetupPreviewEnvironment;
}): Promise<Workspace>;
/**
 * Returns the absolute directory path of the closest ancestor containing node_modules.
 */
declare function findWorkspaceRoot(absoluteFilePath: string): string | null;
interface Workspace {
    rootDirPath: string;
    reader: Reader;
    preview: {
        start(allocatePort?: () => Promise<number>): Promise<Preview>;
    };
    localRpc<E extends RPC<any, any>>(endpoint: E, request: RequestOf<E>): Promise<ResponseOf<E>>;
    dispose(): Promise<void>;
}
interface Preview {
    url(): string;
    stop(options?: {
        onceUnused?: boolean;
    }): Promise<void>;
}

export { Component, ComponentAnalysis, FrameworkPlugin, FrameworkPluginFactory, Preview, SetupPreviewEnvironment, Workspace, createWorkspace, findWorkspaceRoot, setupFrameworkPlugin };
